
#ifndef __RAS_KMSG_H
#define __RAS_KMSG_H

#include <regex.h>
#include <stdint.h>

/**
 * Kernel message tracer related definitions
 */
#define KMSG_TRACE_NUM	"KMSG_TRACE_NUM"
#define KMSG_TRACER_NAME "KMSG_TRACER_NAME"
#define KMSG_TRACER_REGEX "KMSG_TRACER_REGEX"
#define KMSG_TRACER_GROUP_COUNT "KMSG_TRACER_GROUP_COUNT"
#define KMSG_TRACER_GROUP_KEY "KMSG_TRACER_GROUP_KEY"

#define KMSG_TRACE_END	"KMSG_TRACE_END"
#define KMSG_IGNORE_XID	"KMSG_IGNORE_XID"
#define KMSG_LIMIT	"KMSG_LIMIT"

#define NAME_LEN 64
#define BUF_LEN 1024
#define PRINTK_MESSAGE_MAX 2048

extern int kmsg_monitor;
extern int kmsg_trace_end;
extern int kmsg_limit;

struct kmsg_tracer_info {
	char *name;
	char *regex;
	int group_count;
	char **group_key;
	regex_t regex_c;
	regmatch_t *matches;
	union {
		struct {
			int len;
			int ignore_xid[30];
		} xid;
	} info;
};

struct amdgpu_tracer {
	regex_t header;
	regex_t status;
	regex_t addr;
	regex_t misc0;
	regex_t ipid;
	regex_t synd;
};

struct amdgpu_error {
	char timestamp[64];
	int seq, bus, dev, func;
	int tracing;
	uint64_t status;
	uint64_t addr;
	uint64_t misc0;
	uint64_t ipid;
	uint64_t synd;
};

int kmsg_tracer_init(void);
int kmsg_tracer_destroy(void);
int kmsg_match(char *msg);

void amdgpu_tracer_match(char *msg);
int amdgpu_tracer_destroy(void);
int amdgpu_tracer_init(void);

#endif
