
/* SPDX-License-Identifier: GPL-2.0-or-later */

/*
 * Copyright (C) 2025 Alibaba Inc
 */


#include <dlfcn.h>        
#include <stdio.h>        
#include "./contrib/nvml.h"

typedef const char* (*my_nvmlErrorString_p)(nvmlReturn_t result);
typedef nvmlReturn_t (*my_nvmlShutdown_p)(void);
typedef nvmlReturn_t (*my_nvmlEventSetCreate_p)(nvmlEventSet_t *set);
typedef nvmlReturn_t (*my_nvmlDeviceRegisterEvents_p)(nvmlDevice_t device, unsigned long long eventTypes, nvmlEventSet_t set);
typedef nvmlReturn_t (*my_nvmlDeviceGetSupportedEventTypes_p)(nvmlDevice_t device, unsigned long long *eventTypes);
typedef nvmlReturn_t (*my_nvmlEventSetFree_p)(nvmlEventSet_t set);
typedef nvmlReturn_t (*my_nvmlInit_p)(void);
typedef nvmlReturn_t (*my_nvmlDeviceGetCount_p)(unsigned int *deviceCount);
typedef nvmlReturn_t (*my_nvmlDeviceGetHandleByIndex_p)(unsigned int index, nvmlDevice_t *device);
typedef nvmlReturn_t (*my_nvmlDeviceGetPciInfo_p)(nvmlDevice_t device, nvmlPciInfo_t *pci);
typedef nvmlReturn_t (*my_nvmlEventSetWait_p)(nvmlEventSet_t set, nvmlEventData_t * data, unsigned int timeoutms);

my_nvmlErrorString_p my_nvmlErrorString;
my_nvmlShutdown_p my_nvmlShutdown;
my_nvmlEventSetCreate_p my_nvmlEventSetCreate;
my_nvmlDeviceRegisterEvents_p my_nvmlDeviceRegisterEvents;
my_nvmlDeviceGetSupportedEventTypes_p my_nvmlDeviceGetSupportedEventTypes;
my_nvmlEventSetFree_p my_nvmlEventSetFree;
my_nvmlInit_p my_nvmlInit;
my_nvmlDeviceGetCount_p my_nvmlDeviceGetCount;
my_nvmlDeviceGetHandleByIndex_p my_nvmlDeviceGetHandleByIndex;
my_nvmlDeviceGetPciInfo_p my_nvmlDeviceGetPciInfo;
my_nvmlEventSetWait_p my_nvmlEventSetWait;

static int my_nvml_setup(void* handle) 
{
	my_nvmlShutdown = (my_nvmlShutdown_p)dlsym(handle, "nvmlShutdown");                                 
	if (!my_nvmlShutdown) {                                 
		printf("Failed to load nvmlShutdown: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlEventSetCreate = (my_nvmlEventSetCreate_p)dlsym(handle, "nvmlEventSetCreate");                                 
	if (!my_nvmlEventSetCreate) {                                 
		printf("Failed to load nvmlEventSetCreate: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlDeviceRegisterEvents = (my_nvmlDeviceRegisterEvents_p)dlsym(handle, "nvmlDeviceRegisterEvents");                                 
	if (!my_nvmlDeviceRegisterEvents) {                                 
		printf("Failed to load nvmlDeviceRegisterEvents: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlDeviceGetSupportedEventTypes = (my_nvmlDeviceGetSupportedEventTypes_p)dlsym(handle, "nvmlDeviceGetSupportedEventTypes");                                 
	if (!my_nvmlDeviceGetSupportedEventTypes) {                                 
		printf("Failed to load nvmlDeviceGetSupportedEventTypes: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlEventSetFree = (my_nvmlEventSetFree_p)dlsym(handle, "nvmlEventSetFree");                                 
	if (!my_nvmlEventSetFree) {                                 
		printf("Failed to load nvmlEventSetFree: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlInit = (my_nvmlInit_p)dlsym(handle, "nvmlInit");                                 
	if (!my_nvmlInit) {                                 
		printf("Failed to load nvmlInit: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlDeviceGetCount = (my_nvmlDeviceGetCount_p)dlsym(handle, "nvmlDeviceGetCount");                                 
	if (!my_nvmlDeviceGetCount) {                                 
		printf("Failed to load nvmlDeviceGetCount: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlDeviceGetHandleByIndex = (my_nvmlDeviceGetHandleByIndex_p)dlsym(handle, "nvmlDeviceGetHandleByIndex");                                 
	if (!my_nvmlDeviceGetHandleByIndex) {                                 
		printf("Failed to load nvmlDeviceGetHandleByIndex: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlDeviceGetPciInfo = (my_nvmlDeviceGetPciInfo_p)dlsym(handle, "nvmlDeviceGetPciInfo");                                 
	if (!my_nvmlDeviceGetPciInfo) {                                 
		printf("Failed to load nvmlDeviceGetPciInfo: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlEventSetWait = (my_nvmlEventSetWait_p)dlsym(handle, "nvmlEventSetWait");                                 
	if (!my_nvmlEventSetWait) {                                 
		printf("Failed to load nvmlEventSetWait: %s\n", dlerror());                                 
		return -1;                                 
	}
	my_nvmlErrorString = (my_nvmlErrorString_p)dlsym(handle, "nvmlErrorString");          
	if (!my_nvmlErrorString) {          
		printf("Failed to load nvmlErrorString: %s\n", dlerror());          
		return -1;          
	}
	return 0;
}

static const char* my_nvmlEventTypeString(unsigned long long type) 
{         

	switch (type) {         
	case nvmlEventTypeNone: return "None";
	case nvmlEventTypeSingleBitEccError: return "SingleBitEccError";
	case nvmlEventTypeDoubleBitEccError: return "DoubleBitEccError";
	case nvmlEventTypePState: return "PState";
	case nvmlEventTypeXidCriticalError: return "XidCriticalError";
	case nvmlEventTypeClock: return "Clock";
	case nvmlEventTypePowerSourceChange: return "PowerSourceChange";
	case nvmlEventTypeSingleBitEccErrorStorm: return "SingleBitEccErrorStorm";
	case nvmlEventTypeDramRetirementEvent: return "DramRetirementEvent";
	case nvmlEventTypeDramRetirementFailure: return "DramRetirementFailure";
	case nvmlEventTypeNonFatalPoisonError: return "NonFatalPoisonError";
	case nvmlEventTypeFatalPoisonError: return "FatalPoisonError";         
	default: return "Unknown";         
	}         
	return "Unknown";         
}
